1) Clone and build faster-rcnn.pytorch from [this repository](https://github.com/jwyang/faster-rcnn.pytorch). We use this code as a base for our experiments. The following steps document what is needed to get it running.
    
    - Clone repository and checkout pytorch-1.0 branch
    ```bash
    git clone "https://github.com/jwyang/faster-rcnn.pytorch"
    cd faster-rcnn.pytorch
    git checkout pytorch-1.0
    ```
    
    - Install requirements
    ```bash
    pip install -r requirements.txt
    ```
    
    - Build
    ```bash
    cd lib
    python setup.py build develop
    ```

    - Replace pycocotools
    ```bash
    cd ../..
    git clone "https://github.com/cocodataset/cocoapi"
    cd cocoapi/PythonAPI

    make all

    cd ../../faster-rcnn.pytorch/lib
    rm -rf pycocotools
    cp -r ../../cocoapi/PythonAPI/pycocotools ./
    cd ..
    ```

    - Download trained models (ours)
    ```bash
    wget "https://www.dropbox.com/s/dnxsdhkhcj9jvn0/models.zip"
    unzip models.zip
    ```

    - Get datasets (PASCAL trainval and COCO train datasets can be skipped if training is not needed)
    ```bash
    mkdir data
    cd data

    wget "http://host.robots.ox.ac.uk/pascal/VOC/voc2007/VOCtrainval_06-Nov-2007.tar"
    wget "http://host.robots.ox.ac.uk/pascal/VOC/voc2007/VOCtest_06-Nov-2007.tar"
    wget "http://host.robots.ox.ac.uk/pascal/VOC/voc2007/VOCdevkit_08-Jun-2007.tar"
    tar -xvf VOCtrainval_06-Nov-2007.tar
    tar -xvf VOCtest_06-Nov-2007.tar
    tar -xvf VOCdevkit_08-Jun-2007.tar
    mv VOCdevkit VOCdevkit2007

    mkdir coco
    cd coco

    wget "http://images.cocodataset.org/zips/train2014.zip"
    wget "http://images.cocodataset.org/zips/val2014.zip"
    wget "http://images.cocodataset.org/annotations/annotations_trainval2014.zip"
    wget "https://dl.dropboxusercontent.com/s/o43o90bna78omob/instances_minival2014.json.zip"
    wget "https://dl.dropboxusercontent.com/s/s3tw5zcg7395368/instances_valminusminival2014.json.zip"
    unzip train2014.zip
    unzip val2014.zip
    unzip annotations_trainval2014.zip
    unzip instances_minival2014.json.zip
    unzip instances_valminusminival2014.json.zip
    mkdir images
    mv train2014 images/
    mv val2014 images/
    mv instances_minival2014.json annotations/
    mv instances_valminusminival2014.json annotations/

    cd ../..
    ```

    - Test on PASCAL VOC (with our trained models)
    ```bash
    python test_net.py --dataset pascal_voc --net vgg16 \
                       --checksession 1 --checkepoch 6 --checkpoint 10021 \
                       --cuda
    ```

    The results should be same as gNMS 300.

2) Add our modifications to the codebase. Move the `faster-rcnn.pytorch.dpp.zip` file into `faster-rcnn.pytorch` directory and follow the next few steps. Before running `make` kindly ensure the `Makefile` has correct paths for Python, Boost, and Eigen3 libraries. If not using python 3.7, the linker directives `-lboost_python37`, `-lboost_numpy37`, `-lpython3.7m` will also need to changed accordingly.

    - Unzip and overwrite existing files
    ```bash
    unzip -o faster-rcnn.pytorch.dpp.zip
    ```

    - Build DPP code
    ```bash
    cd lib/model

    make

    cd ../..
    ```

3) Test DPP based Faster RCNN. 

    - PASCAL VOC
    ```bash
    python test_net.py --dataset pascal_voc --net vgg16 \
                       --checksession 1 --checkepoch 6 --checkpoint 10021 \
                       --use_dpp --dpp_alpha 5 \
                       --cuda
    ```

    - COCO
    ```bash
    python test_net.py --dataset coco --net vgg16 \
                       --checksession 1 --checkepoch 6 --checkpoint 58632 \
                       --use_dpp --dpp_alpha 5 \
                       --cuda
    ```

    - To set `k` for dpp change `line 196` in `faster-rcnn.pytorch/lib/model/utils/config.py`
    ```python
    __C.TEST.RPN_POST_NMS_TOP_N = 300
    ```